/*
  Copyright Red Hat, Inc. 2005

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2, or (at your option) any
  later version.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; see the file COPYING.  If not, write to the
  Free Software Foundation, Inc.,  675 Mass Ave, Cambridge, 
  MA 02139, USA.
*/
/*
 * Author: Stanko Kupcevic <kupcevic@redhat.com>
 */


#include "Server.h"
#include "ricci_defines.h"

#include <sys/types.h>
#include <unistd.h>
extern "C" {
  void daemon_init(char *prog);
}



#include <iostream>


using namespace std;


int main(int argc, char** argv)
{
  bool foreground=false, debug=false;
  unsigned int uid = 0;
  
  int rv;
  while ((rv = getopt(argc, argv, "fdu:")) != EOF)
    switch (rv) {
    case 'd':
      debug = true;
      break;
    case 'f':
      foreground = true;
      break;
    case 'u':
      sscanf(optarg, "%d", &uid);
      break;
    default:
      break;
    }
  
  if (geteuid() != 0) {
    cout << "Only root can run this" << endl;
    return 1;
  }
  
  try {
    ServerSocket serv_sock(RICCI_SERVER_PORT);
    
    if (!foreground)
      daemon_init(argv[0]);
    
    if (uid != 0)
      // change user
      if (setreuid(uid, uid)) {
	cout << "unable to change uid to " << uid << endl;
	return 1;
      }
    
    Server server(serv_sock);
    server.run();
  } catch ( String e ) {
    cout << "exception: " << e << endl;
    return 1;
  }
  return 0;
}
